system.require('std/core/bind.em');
system.require('std/core/repeatingTimer.em');


if(typeof(state) === 'undefined')
	state = {};

//state.defaultTimeout = 5 /* * u.s */;

//Class used for interacting with the bank.  Provides GUI and wraps message sending.

//To instantiate class requires the visible of the bank you are connecting to.  If created on
//an entity without a simulator defined then the GUIs do not open.  Should handle that correctly
//but should check into that.

state.Connection = function (service) {
	system.print('Setting up new state.');
	this.timeout = 100;
	
	this.service = service;
	this.getTrades = false;
	this.transaction = 1;
	//Login GUI for the bank.
	var ui = @sirikata.ui(
		'bankLogin',
		function() {
		  
			$('<div id="bankLogin-ui" title="Bank of Elliot">' +
				'  <div id="loginResult"/>' +	
              '  <input value="Username" id="usernameb"></input>' +		
              '  <input value="Password" id="passwordb"></input>' +					  
			  '  <button id="login-button">Login</button>' +
			  '  <button id="create-button">Create New Account</button>' +
			  
			  '</div>').appendTo('body');

			var window = new sirikata.ui.window(                  
			   "#bankLogin-ui",
			   {
				  width: 100,
				  height: 'auto'
			   }
			);
			window.show();

			sirikata.ui.button('#login-button').click(loginBank);  
			sirikata.ui.button('#create-button').click(createBank); 
            //var username = 'none';
			//var password = 'none';
			function loginBank() {	
                var usernameb = ($('#usernameb').val());	
				var passwordb = ($('#passwordb').val());
				window.toggle();
				sirikata.event('login', usernameb, passwordb);
			}
			function createBank() {	
                var usernameb = ($('#usernameb').val());	
				var passwordb = ($('#passwordb').val());
				window.toggle();
				sirikata.event('create', usernameb, passwordb);
			}
			loginStatus = function(message) {
				window.toggle();
				$( "#loginResult" ).text(message);
			};
			
		}
	);@;
	//Wraps account creation.

	bankCreate = function(username, password)  {
	   system.__debugPrint("\n\n TEST" + username + '\n\n');
	   this.create(username,password,std.core.bind(loginStatus,this));
	   //system.timeout(2,std.core.bind(this.openBankGUI,this));
	};
	
	//Wraps account login.
	bankLogin = function(username, password) {
	   this.login(username,password, false, std.core.bind(loginStatus,this));
	   //system.timeout(2,std.core.bind(this.openBankGUI,this));
	};
	
	//Callback passed to account creation and login through the GUI.  Either opens up account
	//balance and trade window if success or else reopens login window with error message if failure.
	loginStatus = function(message) {
		if (typeof(message.error) === 'undefined') {
			//If pending trades have not yet been retrieved then ask for them.  Otherwise just open GUI.
			if (this.getTrades) {
				system.__debugPrint('Getting trades in login cb');
				{action: 'getTrades'} >>this.service >> [];
			} else
				this.getTrades = true;
			this.openBankGUI();
		} else {
			//Reopens login message with error.
			this.loginGUI.call('loginStatus','Login failed: ' +message.error);
		}
	};
	
	//GUI creation callback.  Creates GUI and binds login and create events.
	GUIfunc = function (gui) {
		this.loginGUI = gui;
		gui.bind("login", std.core.bind(bankLogin,this));
        gui.bind("create", std.core.bind(bankCreate,this));
	};
	
	//If simulator is defined then create the GUI.
	if (typeof(simulator) !=='undefined')
		simulator._simulator.addGUITextModule("bankLogin", ui, std.core.bind(GUIfunc, this));
	
	
	
	/***************************************************************/
	//Trade UI
	
	//GUI to be opened upon trade proposal.
	var tui = @sirikata.ui(
			'trade',
			function() {
				$('<div id="trade-ui" title="Trade Request">' +
				  '  <div id="Tradeto"/>' +	
				  '  <div id="Tradefor"/>' +					  
				  '  <button id="agree-button">Yes!</button>' +
				  '  <button id="decline-button">No!</button>' +
				  '</div>').appendTo('body');

				var window = new sirikata.ui.window(                  
				   "#trade-ui",
				   {
				      autoOpen: false,
					  width: 100,
					  height: 'auto'
				   }
				);
				//window.show();

				sirikata.ui.button('#agree-button').click(acceptTrade);  
				sirikata.ui.button('#decline-button').click(declineTrade);   
				function acceptTrade() {
					window.toggle();				
					sirikata.event('trade', 'accept');
				}
				function declineTrade() {
					window.toggle();					
					sirikata.event('trade', 'decline');
				}
				//$( "#Tradeto" ).text('Test');
				
				setTradeText = function(str) {
				  $( "#Tradeto" ).text(str);
				};
				openTrade = function(str) {
				  window.toggle();
				  setTradeText(str);
				};
				
			}
	);@;
	
	//Sends reply to bank.
	function bankReply(cmd) {
	   var rmsg = new Object();
	   rmsg.resp = cmd;
	   rmsg >> this.service >> [];
	  
	  //Gets the bank balance after the trade to update balance on GUI.
	   this.get({type: 'money'}, std.core.bind(this.updateBal, this));
	   
	}
	
	//Callback for trade GUI creation.  Gets pending trades from bank if not already retrieved.  
	var TGUIfunc = function (gui) {
		system.print('Setting up GUI.');
		gui.bind("trade", std.core.bind(bankReply,this));
		this.tradeGUI = gui;
		system.__debugPrint('Getting trades from bank');
		if (this.getTrades) {
			system.__debugPrint('Getting trades in tguifunc');
			
			{action: 'getTrades'} >>this.service >> [];
		} else
			this.getTrades = true;
		//gui.call('setTradeText', tradeStr);
	};
	
	//Opens trade window with requested text.
	openTrade = function(msg, sender) {
	  system.__debugPrint('Got a trade request. \n\n');
	  this.tradeGUI.call('openTrade',msg.tradeRequest);
	};
	//system.print(tradeStr + ' is the trade string.');
	if (typeof(simulator) !=='undefined')
		std.core.bind(openTrade, this) << {'tradeRequest'::} << this.service;
	
	if (typeof(simulator) !=='undefined')
		simulator._simulator.addGUITextModule("trade", tui, std.core.bind(TGUIfunc, this));
	
	//Function that pings bank on a timer of 5 SECONDS to show that you have not disconnected 
	//from world and should stay logged in.  Bank checks for message every 10 seconds.
	this.pingBank = function() {
		system.__debugPrint('PINGING BANK');
		{action: 'updatePing'} >> this.service >> [];
	};
	this.openBankGUI = function () {
		//Creates timer pinging bank.
		this.pingTimer =new std.core.RepeatingTimer(5, std.core.bind(this.pingBank, this));
		
		//GUI for the main bank interface.
	    var bui = @sirikata.ui(
			'bank',
			function() {
			  
				$('<div id="bank-ui" title="Bank of Elliot">' +
				  '  <div id="Balance"/>' +	
				  '  <div id="OwnedPres"/>' +	
				  '  <input value="Trade: Username" id="tradeUsername"></input>' +
				
				  '<div class = "trade-area">' +
				  
				  '</div>' +
				  
				  
				  '  <button id="updateBal-button">Update Balance</button>' +
				  '  <button id="addItem-button">Add to Trade</button>' +
				  
				  '  <button id="resetTrade-button">Reset Trade</button>' +
				  '  <button id="finalTrade-button">Propose Trade</button>' +
				  '  <button id="logoutBank-button">Logout</button>' +
				  '<div class = "trading-area">' +
				  
				  '</div>' +
				 
				  
				  
				  '</div>').appendTo('body');

				sirikata.ui.button('#updateBal-button').click(updateBankBalance); 
				sirikata.ui.button('#addItem-button').click(addToTrade); 
				sirikata.ui.button('#resetTrade-button').click(resetTrade); 
				sirikata.ui.button('#finalTrade-button').click(proposeTrade); 
				sirikata.ui.button('#logoutBank-button').click(logoutBank); 
				
				
				var window = new sirikata.ui.window(                  
				   "#bank-ui",
				   {
					  width: 200,
					  height: 'auto'
				   }
				);
				window.show();
				
				//Array of trade items for each side.
				var myTrades = new Array();
				var theirTrades = new Array();
				/*var myTrades = [{type: 'money', value: 1}];
				var theirTrades = [{type: 'money', value: 50}];*/
				
				function logoutBank() {
				  sirikata.event('bankLogout');
				  window.toggle();
				}
				function updateBankBalance() {
				  sirikata.event('getBal');
				}
				
				//Sends trade proposal event for sending to bank and puts up Pending Approval message.
				function proposeTrade() {
				  sirikata.log('fatal','CALLED PROPOSE');
				  sirikata.log('fatal', 'myTrades:'+myTrades + 'theirTrades:' + theirTrades + 'Username:' + ($('#tradeUsername').val()));
				  sirikata.event('tradeFinal',JSON.stringify(myTrades),JSON.stringify(theirTrades),($('#tradeUsername').val()));
				  myTrades = [];
				  theirTrades = [];
				  $('div.trading-area').html( '');
				  $('div.trade-area').html( 'Pending Approval...');
				  //window.toggle();
				}
				
				
				//Complicated trade stuff.//////////////////////////////////////////////////
				function addToTrade() {
				  $('div.trading-area').html( '  <button id="Money-button">Trade Money</button>'+ '  <button id="Presence-button">Trade Presence</button>' + '  <button id="Service-button">Trade Service</button>');
				  sirikata.ui.button('#Money-button').click(MoneyTrade);
				  sirikata.ui.button('#Service-button').click(ServiceTrade);
				  sirikata.ui.button('#Presence-button').click(PresenceTrade);
				}
				function resetTrade() {
				  myTrades = [];
				  theirTrades = [];
				  $('div.trading-area').html( '');
				  $('div.trade-area').html( '');
				}
				var tradeType;
				function MoneyTrade() {
				  tradeType = 'money';
				  $('div.trading-area').html( '  <input value="Money Amount" id="moneyAmount"></input>' +'  <button id="addMine-button">Add to my side</button>'+ '  <button id="addTheirs-button">Add to their side</button>');
				  sirikata.ui.button('#addMine-button').click(myTrade);
				  sirikata.ui.button('#addTheirs-button').click(theirTrade);
				}
				
				function ServiceTrade() {
				  tradeType = 'service';
				  $('div.trading-area').html( 'Input a string representing the service to be provided.' + '  <input value="" id="serviceStr"></input>' +'  <button id="addMine-button">Add to my side</button>'+ '  <button id="addTheirs-button">Add to their side</button>');
				  sirikata.ui.button('#addMine-button').click(myTrade);
				  sirikata.ui.button('#addTheirs-button').click(theirTrade);
				}
				
				function PresenceTrade() {
				  tradeType = 'pres';
				  $('div.trading-area').html( 'Please select the presence to be traded.  Then choose which side of the trade to add it to.' + '  <button id="addMine-button">Add to my side</button>'+ '  <button id="addTheirs-button">Add to their side</button>');
				  sirikata.ui.button('#addMine-button').click(myTrade);
				  sirikata.ui.button('#addTheirs-button').click(theirTrade);
				}
				var traded;
				function myTrade() {
				  
				  if (tradeType == 'pres') {
					sirikata.event('getSelected', 'mine');
				  } else if (tradeType === 'money'){
					var trade = {type: 'money', value: ($('#moneyAmount').val())};
					
					myTrades.push(trade);
					$('div.trade-area').html(function(index,old) {return old+'<br/> You will trade:'+ ($('#moneyAmount').val())+ ' neo shekels';});	
				  } else if (tradeType === 'service') {
				    var trade = {type: 'string', value: ($('#serviceStr').val())};
					myTrades.push(trade);
					$('div.trade-area').html(function(index,old) {return old+'<br/> You will provide:'+ ($('#serviceStr').val());});	
				  }
				  $('div.trading-area').html( '');
				}
				
				function theirTrade() {
				
				  if (tradeType === 'pres') {
					sirikata.event('getSelected', 'theirs');
				  } else if (tradeType === 'money') {
					var trade = {type: 'money', value: ($('#moneyAmount').val())};
					
					theirTrades.push(trade);
					$('div.trade-area').html(function(index,old) {return old+ '<br/> They will trade:'+ ($('#moneyAmount').val())+ ' neo shekels';});	
				  } else if (tradeType === 'service') {
					var trade = {type: 'string', value: ($('#serviceStr').val())};
					myTrades.push(trade);
					$('div.trade-area').html(function(index,old) {return old+'<br/> They will provide:'+ ($('#serviceStr').val());});	
				  }
				  $('div.trading-area').html( '');
				}
				//If a presence trade then an event has to be sent to Emerson to get the selected visible.
				returnSelected = function(selected, side) {
				  if (side == 'mine') {
					var trade = {type:'presence', value: selected};
					
					myTrades.push(trade);
					$('div.trade-area').html(function(index,old) {return old + '<br/> You will trade presence:'+ selected;});	
				  } else if (side == 'theirs') {
					var trade = {type:'presence', value: selected};
					
					theirTrades.push(trade);
					$('div.trade-area').html(function(index,old) {return old + '<br/> They will trade presence:'+ selected;});	
				  }
				};
				updateBalance = function(val) {
					$( "#Balance" ).text('Your Balance: ' +val + ' neo shekels');
				};
				tradeUpdate = function(result) {
				  $('div.trade-area').html( 'Trade '+result+'.');
				};
				openBankGUI = function() {
					window.toggle();
				}
				//////////////////////////////////////////////////////////////////////////
			}
		);@;



		/*updateBal = function(val) {
		  this.bankGUI.call('updateBalance',val);
		};*/
		//Returns visible selected.
		getSelected = function(side) {
		  this.bankGUI.call('returnSelected',simulator._selected.toString(), side);
		};
		tradeResponse = function(result) {
		  this.bankGUI.call('tradeUpdate',result);
		  this.get({type: 'money'}, std.core.bind(this.updateBal, this));
		};
		giveResponse = function (result) {
		  tradeResponse(result.result);
		};
		tradeFinalize = function(myTrades, theirTrades, username) {
		  var mine = JSON.parse(myTrades);
		  var theirs = JSON.parse(theirTrades);
		  system.__debugPrint("\n\n TESTTRADE \n\n");
		  system.__debugPrint("\n\n TESTTRADE\n\n" + username + myTrades+'\n\n');
		  if (theirs.length == 0) {
			//If you are not requesting anything and just giving it instead uses a more simple
			//give request.
			this.give(mine,username, std.core.bind(tradeResponse,this));
			return;
		  }
		  if (mine.length == 0) {
			//If you are just requesting without giving it is probably not a trade the other
			//person would have any interest in.  So it just rejects it.  This might want to change
			//If there is a use case I haven't yet thought of.
			tradeResponse('failed');
			return;
		  }
		  this.trade(mine, theirs, username, std.core.bind(tradeResponse,this), std.core.bind(tradeResponse,this), false);
		  system.__debugPrint("\n\n TESTTRADE\n\n" + username + mine+'\n\n');
		};
		getNewBalance = function() {
		  this.get({type: 'money'}, std.core.bind(this.updateBal, this));
		}
		logout = function() {
			this.pingTimer.suspend();
			
			{action: 'logout'} >> this.service >> [std.core.bind(this.openLogin,this),5,function(){}];
		};
		
		//Bank GUI creation callback, binds a lot of stuff and gets the current bank balance.
		var bankGUIfunc = function (gui) {
			this.bankGUI = gui;
			gui.bind("getSelected", std.core.bind(getSelected,this));
			gui.bind("tradeFinal", std.core.bind(tradeFinalize,this));
			gui.bind("bankLogout", std.core.bind(logout,this));
			gui.bind("getBal", std.core.bind(getNewBalance,this));
			this.get({type: 'money'}, std.core.bind(this.updateBal, this));
		};
		
		if (typeof(this.bankGUI) === 'undefined' &&  typeof(simulator) !=='undefined')
			simulator._simulator.addGUITextModule("bank", bui, std.core.bind(bankGUIfunc, this));
		else {
			if (typeof(simulator) !=='undefined') return;
			this.get({type: 'money'}, std.core.bind(this.updateBal, this));
			this.bankGUI.call('openBankGUI');
		}
	};
	this.updateBal = function(val) {
	  
	  this.bankGUI.call('updateBalance',val);
	};
	this.openLogin = function(msg) {
		if (typeof(msg.error) === 'undefined') {
			this.loginGUI.call('loginStatus',msg.success);
		}	
	};
	this.login =  function(username, password, create, cb) {
		//If callback is undefined then just a blank function is used.
		if (typeof(cb) ==='undefined')
			cb = function () {};
		if(typeof(create) === 'undefined')
			create = false;
		
		this.username = username;
		this.password = password;
		
		var errorResponse = function(message, sender) {
			throw('Error response "' + message.error +
					'" received from state service ' + sender.toString());
		};
		
		var printResponse = function(message, sender) {
			system.print(message.print);
		};
		
		var loginConfirm = function(message, sender) {
			system.__debugPrint('GOT RESPONSE');
			cb(message);
			
			};
		var loginFailed = function() {
		system.__debugPrint('GOT RESPONSE');
			cb(message);
			/*throw('Login for user ' + username +
					' failed: message timed out');*/
		};
		
		var action = (create ? 'create' : 'login');
		system.__debugPrint('Actually sending out message');
		{action: action, username: this.username, password: this.password} >>
				this.service >> [std.core.bind(loginConfirm,this), this.timeout, std.core.bind(loginFailed,this)];
		system.print(action);
		system.print(username);
		system.print(password);
	
		errorResponse << [{'error'::}] << this.service;
		printResponse << [{'print'::}] << this.service;
	};
	
	//Wraps login with the creation flag true.
	this.create = function(username, password, cb) {
	if (typeof(cb) ==='undefined')
			cb = function() {};
		system.__debugPrint('creating account');
		this.login(username, password, true, cb);
	};
	
	this.logout = function() {
		{action: 'logout', username: this.username} >> this.service >> [];
	};
	
	
	//get is used to find a value from the bank.  In this case the balance is the most frequently used.
	//
	this.get = function(key, callback) {
	
		this.callback = callback;
		
		var onResponse = function(message, sender) {
			if('error' in message)
				return;
			
			if(!('value' in message))
				throw('Bad mesage: response for value "' + key +
						'" did not have a value field');
			
			callback(message.value);
		};
		
		var onNoResponse = function() {
			throw('Could not get value "' + key + '": message timed out');
		};
		
		{action: 'get', key: key} >> this.service >>
				[onResponse, this.timeout, onNoResponse];
	};
	
	this.give = function(mine, recipient, callback) {
		var onResponse = function(message, sender) {
			if(typeof(callback) !== 'undefined')
				callback(message.result);
		};
		
		var onNoResponse = function() {
			throw('Could not give "' + std.core.pretty(mine) + '" to ' +
					std.core.pretty(recipient) + ': message timed out');
		};
		
		{action: 'give', what: mine, to: recipient} >> this.service >>
				[std.core.bind(onResponse,this), this.timeout, onNoResponse];
	};
	
	//Registers a visible as owned.
	this.register = function(pres, callback) {
	  var onResponse = function(message, sender) {
			if(!('error' in message) && typeof(callback) !== 'undefined')
				callback();
		};
		
		var onNoResponse = function() {
			throw('Could not register "' + std.core.pretty(pres) + '"');
		};
		
		{action: 'registerObject', object: pres.toString()} >> this.service >>
				[onResponse, this.timeout, onNoResponse];
	  
	};
	
	//The trade wrapper.  Uses a little complex message stufff.
	this.trade= function(mine, yours, you, onSuccess, onFailure, asynchr) {
		onSuccessMessage = function(message, sender) {
			if(typeof(onSuccess) !== 'undefined')
				onSuccess('approved');
		};
		
		onFailureMessage = function(message, sender) {
			if(typeof(onFailure) !== 'undefined') {
			    if (typeof(message.result) == 'undefined') {
				  onFailure('failed');
				} else
				  onFailure(message.result);
			}
		};
		
		var onResponse = function(message, sender) {
			if(!('seqNoID' in message) || !('id' in message))
				throw('Response "' + std.core.pretty(message) + 'to trade "' +
						std.core.pretty(mine) + '" to ' + you + ' for "' +
						std.core.pretty(yours) + '" is incorrectly formatted -- '+
						'must contain seqNo and id.');
		    if (typeof(message.status) != 'undefined')
			  onSuccess(message.status);
			if (typeof(message.asynch)!== 'undefined')
				this.bankGUI.call('tradeUpdate',message.asynch);
			onSuccessMessage << [{'seqNoID':message.seqNoID:}, {'id':message.id:},
					{'status':'success':}] << this.service;
			onFailureMessage << [{'seqNoID':message.seqNoID:}, {'id':message.id:},
					{'status':'failure':}] << this.service;
		};
		
		var onNoResponse = function() {
			throw('Could not give "' + std.core.pretty(mine) + '" to ' +
					std.core.pretty(recipient) + ': message timed out');
		};
		//The first reply uses the makeReply method and essentially just confirms that the trade has been
		//registered, checked as valid and sent off to the recipient.
		{action: 'trade', what: mine, to: you, forTrade: yours, asynch: asynchr} >>
				this.service >> [std.core.bind(onResponse, this), this.timeout, std.core.bind(onNoResponse,this)];
	};

};